"""
ampcontrol is a basic python interface for communication with
multichannel amplifiers and recievers via the RS-232
communication port.

Copyright (C) 2009  Vidar Waagbo <vidar@waagbo.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.


TODO:
    
    [ ] Read input from Rotel (Rotel -> PC)
    [ ] Implement all DSP settings
    [ ] Better exception handling?
    [ ] BUG: Mute is not working for zone 1. Probably need a special zone exception.


"""

import sys
import serial
import binascii
from optparse import OptionParser

class Control:
    
    def __init__(self, port = None):
        if(port == None):
            self.amp = serial.Serial("/dev/ttyUSB0")
        else:
            self.amp = serial.Serial(port)

        self.amp.baudrate = 38400
    
    # INTERNAL ROUTINES
    def checksum(self, string):
        
        # Create an ascii list of hex values. Listed as bytes.
        temp = list(string)
        bytes_txt = [temp[0]+temp[1], temp[2]+temp[3], temp[4]+temp[5], temp[6]+temp[7], temp[8]+temp[9]]

        # Sum all the ascii bytes + 2 (Rotel checksum algorithm)
        sum = 2
        for i in bytes_txt:
            sum = sum + int(i,16)
        
        # Return the least significant byte as a hex string =) 
        # Exceptions added for Rotel reserved bytes FD and FE
        cksum = list(hex(sum))[-2] + list(hex(sum))[-1]
        if(cksum.upper() == "FD"):
            return "FD00"
        elif(cksum.upper() == "FE"):
            return "FD01"
        else:
            return cksum

    def get_zone_key(self, zone):
        if(zone == None or zone == 0 or zone == 1):
            zone_key = "10" # Should perhaps be 14 to be poltical correct,
                            # but 10 allows Multi Channel input and stuff.
        elif(zone == 2):
            zone_key = "16"
        elif(zone == 3):
            zone_key = "17"
        elif(zone == 4):
            zone_key = "18"
        else:
            raise Exception("Invalid zone")

        return zone_key

    def custom_command(self, command):
        self.amp.write(binascii.a2b_hex(command))

    def send_command(self, zone_key, command_key):
        command = "FE03A2" + zone_key + command_key
        command = command + self.checksum(command)
        # print(command) # For debug use. Prints command sent to receiver.
        self.amp.write(binascii.a2b_hex(command))

    def get_source_key(self, source):
        if(source.upper() == "CD" or source == "1"):
            source_key = "02"
        elif(source.upper() == "TUNER" or source == "2"):
            source_key = "03"
        elif(source.upper() == "TAPE" or source == "3"):
            source_key = "04"
        elif(source.upper() == "V1" or source.upper() == "VIDEO1" or source == "4"):
            source_key = "05"
        elif(source.upper() == "V2" or source.upper() == "VIDEO2" or source == "5"):
            source_key = "06"
        elif(source.upper() == "V3" or source.upper() == "VIDEO3" or source == "6"):
            source_key = "07"
        elif(source.upper() == "V4" or source.upper() == "VIDEO4" or source == "7"):
            source_key = "08"
        elif(source.upper() == "V5" or source.upper() == "VIDEO5" or source == "8"):
            source_key = "09"
        elif(source.upper() == "MULTI" or source == "9"):
            source_key = "15"
        elif(source.upper() == "SOURCE" or source == "10"):
            source_key = "6B"
        else:
            raise Exception("Invalid source")

        return source_key

    # SOURCE MANAGEMENT
    def toggle_party_mode(self):
        self.amp.write(binascii.a2b_hex("FE03A2106E23"))

    def set_source(self, source, zone = None):
        if(source.upper() == "SOURCE" or source == "10"):
            raise Exception("SOURCE is only available for recording")
        if((source.upper() == "MULTI" or source == "9") and zone >= 2):
            raise Exception("MULTI is only available for main zone")

        # Set the correct hex value for the zone:
        zone_key = self.get_zone_key(zone)
        
        # Set the correct hex value for the input device:
        source_key = self.get_source_key(source)      

        # Generate and send the command:
        self.send_command(zone_key, source_key)

    def set_record_source(self, source):
        if(source.upper() == "MULTI" or source == "9"):
            raise Exception("MULTI is only available for playback")

        zone_key = "15"
        source_key = self.get_source_key(source)
        self.send_command(zone_key, source_key)


    # POWER MANAGEMENT
    def set_power(self, state, zone = None):
        
        # Special zone key for main power (10 not 14)
        if(zone == None or zone == 0 or zone == 1):
            zone_key = "10"
        else:
            zone_key = self.get_zone_key(zone)

        if(state == True):
            power_key = "4B"
        elif(state == False):
            power_key = "4A"
        else:
            raise Exception("Invalid power state")

        self.send_command(zone_key, power_key)

    def complete_power_off(self):
        self.send_command("10", "71")

    # VOLUME MANAGEMENT
    def set_volume(self, amount, zone = None):
        
        # "Volume Direct" commands has its own zone keys:
        if(zone == None or zone == 0 or zone == 1):
            zone_key = "30"
        elif(zone == 2):
            zone_key = "32"
        elif(zone == 3):
            zone_key = "33"
        elif(zone == 4):
            zone_key = "34"
        else:
            raise Exception("Invalid zone")

        # Cheap workaround to allow bytevalues below 0x10
        if(amount < 16):
            volume = "0" + hex(amount).split("x")[1]
        else:
            volume = hex(amount).split("x")[1]

        self.send_command(zone_key, volume)

        #cksum = self.checksum("FE03A230" + volume)
        #self.amp.write(binascii.a2b_hex("FE03A230" + volume + cksum))
    
    # Bug: There is a problem increasing volume with over 40...
    # this does also apply to volume_down.
    def volume_up(self, amount, zone = None):
        
        for i in xrange(amount):
            self.send_command(self.get_zone_key(zone), "00")

    # Bug: There is a problem decreasing volume with over 40..
    def volume_down(self, amount, zone = None):

        for i in xrange(amount):
            self.send_command(self.get_zone_key(zone), "01")

    def mute(self, zone = None):
        self.send_command(self.get_zone_key(zone), "1E")

    def mute_on(self, zone = None):
        self.send_command(self.get_zone_key(zone), "6C")

    def mute_off(self, zone = None):
        self.send_command(self.get_zone_key(zone), "6D")
    
    # CONNECTION MANAGEMENT
    def close_connection(self):
        self.amp.close()

